import chimera

def findMAV():
	# locate and return the (presumably only) running instance
	# of MultAlign Viewer
	from MultAlignViewer.MAViewer import MAViewer
	from chimera.extension import manager
	mavs = [inst for inst in manager.instances
					if isinstance(inst, MAViewer)]
	if len(mavs) != 1:
		raise AssertionError("not exactly one MAV")
	return mavs[0]


# read a file named "structList" whose lines have 3 fields:  two PDB files
# and an output file name for the alignment
for line in open("structList", "r"):
	pdb1, pdb2, output = line.strip().split()

	# open the pdb files
	mol1 = chimera.openModels.open(pdb1, type="PDB")[0]
	mol2 = chimera.openModels.open(pdb2, type="PDB")[0]

	# use MatchMaker to get them superimposed
	from MatchMaker import match, CP_BEST
	# match the best-aligning chain of pdb1 onto pdb2, specifying
	# the similarity matrix, alignment algorithm, and the gap
	# open/extend penalties
	match(CP_BEST, (mol1, [mol2]), "BLOSUM-62", "Needleman-Wunsch",
						12, 2, showSequence=True)


	# find the Multalign Viewer that is now showing
	mav = findMAV()

	# improve the superposition by pruning residue pairings that match
	# poorly
	mav.match(mol1, [mol2], iterate=True)

	# find the original (ungapped) sequences corresponding to the
	# gapped sequences being displayed
	seqs = []
	for gapped in mav.seqs:
		for ungapped in gapped.molecule.sequences():
			if gapped.residues[0] in ungapped.residues:
				seqs.append(ungapped)
				break
		else:
			raise AssertionError("Could not find ungapped sequence"
					" for %s, %s" % (gapped.molecule.name,
					gapped.name))
	# dismiss this mav instance
	mav.Quit()

	# use Match->Align to create the superposition-based alignment
	import StructSeqAlign
	from StructSeqAlign.align import match2align
	# use 4.0 angstroms as the distance cutoff for matching, period (".")
	# as the gap character, and don't look for circular permutations
	match2align(seqs, 4.0, None, ".", False)

	# again find the new MAV instance
	mav = findMAV()

	# write out the alignment in MSF format
	# other format choices:  Clustal ALN, Aligned FASTA, Aligned NBRF/PIR,
	#	GCG RSF, and Stockholm
	from MultAlignViewer.output import saverFunc
	out = open(output, "w")
	saverFunc["MSF"](out, mav, mav.seqs, mav.fileMarkups)
	out.close()

	# again quit the MAV
	mav.Quit()

	# close the PDBs
	chimera.openModels.close([mol1, mol2])
raise chimera.ChimeraSystemExit("script finished\n")
